/*
   Copyright (C) 2005 Michael Nelson Moran

   This file is part of OSCL.

   OSCL is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   OSCL is distributed in the hope that it will be useful, but WITHOUT
   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
   License for more details.

   You should have received a copy of the GNU General Public License
   along with OSCL; see the file COPYING.  If not, write to the Free
   Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.

   To negotiate other licensing arrangments/agreements contact
   the copyright holder:

   Michael N. Moran
   5009 Old Field Ct.         mike@mnmoran.org
   Kennesaw, GA, USA 30144    http://mnmoran.org
*/

#include <sys/timer.h>
#include "service.h"
#include "oscl/timer/counter.h"
/*
    unsigned long OsclTimerGetMicrosecondCounter(void) throw();
    unsigned long OsclTimerGetMillisecondCounter(void) throw();
*/

using namespace Oscl::Mt::Itc::Delay;

Part::Part() throw(){
	}

void	Part::tick(unsigned millisecondsSinceLastTick) throw(){
	Req::Api::Delay*	next	= _pending.first();
	if(!next) return;
	returnExpired(millisecondsSinceLastTick);
	}

bool	Part::delayPending() throw(){
	return _pending.first();
	}

void	Part::returnExpired(unsigned millisecondsSinceLastTick) throw(){
	while(true){
		Req::Api::Delay*	next	= _pending.first();
		if(next){
			unsigned long	value	= next->_milliseconds;
			next->_milliseconds	-= millisecondsSinceLastTick;
			if(next->milliseconds)
			if(!(next->milliseconds) || (next->_milliseconds > value)){
				DelayReq*	req	= (DelayReq*)_pending.get()->_owner;
				req->returnToSender();
				continue;
				}
			}
		break;
		}
	}

// Subtract the two timeval structures such that
// x - y == normalizedDiff
static void	diff(	const struct timeval&	x,
					const struct timeval&	y,
					struct timeval&			normalizedDiff
					) throw(){
	signed long long	xusecs;
	signed long long	yusecs;

	// Convert x to a long long representation
	xusecs	= x.tv_sec;
	xusecs	*= 1000000;
	xusecs	+= x.tv_usec;

	// Convert y to a long long representation
	yusecs	= y.tv_sec;
	yusecs	*= 1000000;
	yusecs	+= y.tv_usec;

	// do the math
	xusecs	-= yusecs;

	// Convert diff back to normalized timeval
	normalizedDiff.tv_usec	= (xusecs%1000000);
	normalizedDiff.tv_sec	= (xusecs/1000000);
	}

// Add the two timeval structures such that
// x + y == normalizedSum
static void	sum(	const struct timeval&	x,
					const struct timeval&	y,
					struct timeval&			normalizedSum
					) throw(){
	signed long long	xusecs;
	signed long long	yusecs;

	// Convert x to a long long representation
	xusecs	= x.tv_sec;
	xusecs	*= 1000000;
	xusecs	+= x.tv_usec;

	// Convert y to a long long representation
	yusecs	= y.tv_sec;
	yusecs	*= 1000000;
	yusecs	+= y.tv_usec;

	// do the math
	xusecs	+= yusecs;

	// Convert diff back to normalized timeval
	normalizedDiff.tv_usec	= (xusecs%1000000);
	normalizedDiff.tv_sec	= (xusecs/1000000);
	}

void	Part::initialize() throw(){
	}

void	Part::mboxSignaled() throw(){
	long long	currentTime;
   struct timeval	tv;
   gettimeofday(&tv, 0);
	
	}

void	Part::request(DelayReq& msg) throw(){
	Req::Api::Delay&	timer	= msg._payload._timer;
	Req::Api::Delay*	next;
	Req::Api::Delay*	prev = 0;
	if(!timer._milliseconds){
		msg.returnToSender();
		return;
		}
	timer._owner		= &msg;
	for(	next=_pending.first();
			next;
			prev = next, next = _pending.next(next)
			){
		if(timer._milliseconds < next->_milliseconds){
			next->_milliseconds	-= timer._milliseconds;
			_pending.insertBefore(next,&timer);
			return;
			}
		else {
			timer._milliseconds	-= next->_milliseconds;
			}
		}
	_pending.put(&timer);
	}

void	Part::request(CancelReq& msg) throw(){
	Req::Api::Delay*	timer	= &msg._payload._timerToCancel._payload._timer;
	Req::Api::Delay*	next;
	for(	next=_pending.first();
			next;
			next = _pending.next(next)
			){
		if(timer == next){
			Req::Api::Delay*	nt	= _pending.next(next);
			if(nt){
				nt->_milliseconds	+= next->_milliseconds;
				}
			_pending.remove(timer);
			DelayReq*	req	= (DelayReq*)next->_owner;
			req->returnToSender();
			}
		}
	msg.returnToSender();
	}

